#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <stdio.h>
#include <string.h>

#include "shader.h"
#include "log.h"
#include "src/shader.h"

void init(void) {
	INFO("Compiling Shaders...\n");

	// create and compile vertex shader
	INFO("Compiling Vertex Shader...\n");
	ShaderCompileResult vertexShader = readAndCompileShaderFromFile("src/shaders/vertex.glsl", GL_VERTEX_SHADER);

	// create and compile fragment shader
	ShaderCompileResult fragmentShader = readAndCompileShaderFromFile("src/shaders/fragment.glsl", GL_FRAGMENT_SHADER);

	if (!vertexShader.success) {
		FATAL("Failed to compile Vertex Shader\n");
		exit(1);
	}

	if (!fragmentShader.success) {
		FATAL("Failed to compile Vertex Shader\n");
		exit(1);
	}

	// create and link shader program

	// create triangle buffer

	// create vertex array object

	glClearColor(0.1f, 0.1f, 0.1f, 1.0f);
}

void draw(void) {
	glClear(GL_COLOR_BUFFER_BIT);
}

void framebuffer_size_callback(GLFWwindow* window, int width, int height) {
	glViewport(0, 0, width, height);
}

int main(int argc, char const *argv[])
{

	for (int i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-v") == 0) {
			logLevel = LOG_LEVEL_DEBUG;
		}
	}

	glfwInit();
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	GLFWwindow *window = glfwCreateWindow(800, 600, "CG1", NULL, NULL);
	if (!window) {
		printf("Failed to open window\n");
		glfwTerminate();
		return 1;
	}
	glfwSetFramebufferSizeCallback(window, framebuffer_size_callback);
	glfwMakeContextCurrent(window);

	glewInit();

	init();

	while (!glfwWindowShouldClose(window)) {
		draw();

		glfwSwapBuffers(window);
		glfwPollEvents();
	}

	glfwTerminate();

	return 0;
}