#include "shader.h"
#include "log.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

const char* readFileToMemory(const char* filepath) {
	FILE* f = fopen(filepath, "r");

	if (!f) {
		perror("Failed to open file");
		return NULL;
	}

	// determine file size

	fseek(f, 0L, SEEK_END);
	size_t size = ftell(f);

	rewind(f);

	// allocate buffer
	char* buf = (char*)malloc(size + 1);
	if (!buf) {
		perror("Failed to allocate memory");
		fclose(f);
		return NULL;
	}

	// read file to buffer
	fread(buf, 1, size, f);

	// set null byte at end of buffer
	buf[size] = '\0';

	fclose(f);
	return buf;
}

ShaderCompileResult readAndCompileShaderFromFile(const char* filepath, GLenum shaderType) {
	ShaderCompileResult result = { .success = false, .shader = 0 };  // Initialize result to indicate failure by default

	DEBUG("%s - Reading shader\n", filepath);
	const char* shaderSource = readFileToMemory(filepath);

	if (!shaderSource) {
		FATAL("Could not read shader source file\n");
		return result;
	}

	GLint shaderSize = strlen(shaderSource);

	DEBUG("%s - Creating shader\n", filepath);
	GLuint shader = glCreateShader(shaderType);
	
	DEBUG("%s - Applying shader source\n", filepath);
	glShaderSource(shader, 1, &shaderSource, &shaderSize);

	DEBUG("%s - Compiling shader\n", filepath);
	glCompileShader(shader);

	GLint status;
	glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
	if (!status) {
		FATAL("Error compiling shader:\n");
		GLchar infoLog[1024];
		glGetShaderInfoLog(shader, 1024, NULL, infoLog);
		FATAL("%s", infoLog);

		result.success = false;
	} else {
		DEBUG("%s - Successfully compiled shader\n", filepath);
		result.success = true;
	}

	result.shader = shader; // Return the shader ID regardless of success or failure
	return result; // Return the ShaderCompileResult struct
}